Package: slider
Title: Sliding Window Functions
Version: 0.2.2
Authors@R: 
    c(person(given = "Davis",
             family = "Vaughan",
             role = c("aut", "cre"),
             email = "davis@rstudio.com"),
      person(given = "RStudio",
             role = "cph"))
Description: Provides type-stable rolling window functions over any R data
    type. Cumulative and expanding windows are also supported. For more
    advanced usage, an index can be used as a secondary vector that
    defines how sliding windows are to be created.
License: MIT + file LICENSE
URL: https://github.com/DavisVaughan/slider
BugReports: https://github.com/DavisVaughan/slider/issues
Depends: R (>= 3.3.0)
Imports: ellipsis (>= 0.3.1), glue, rlang (>= 0.4.5), vctrs (>= 0.3.6),
        warp
Suggests: covr, dplyr (>= 1.0.0), knitr, lubridate, rmarkdown, testthat
        (>= 3.0.0)
LinkingTo: vctrs (>= 0.3.6)
VignetteBuilder: knitr
Config/testthat/edition: 3
Encoding: UTF-8
RoxygenNote: 7.1.1
SystemRequirements: C++11
Collate: 'block.R' 'conditions.R' 'hop-common.R' 'hop-index-common.R'
        'hop-index.R' 'hop-index2.R' 'hop.R' 'hop2.R' 'names.R'
        'phop-index.R' 'phop.R' 'slide-index2.R' 'pslide-index.R'
        'slide-period2.R' 'pslide-period.R' 'slide2.R' 'pslide.R'
        'segment-tree.R' 'slide-common.R' 'slide-index-common.R'
        'slide-index.R' 'slide-period-common.R' 'slide-period.R'
        'slide.R' 'slider-package.R' 'summary-index.R'
        'summary-slide.R' 'utils.R' 'zzz.R'
NeedsCompilation: yes
Packaged: 2021-07-01 19:27:53 UTC; davis
Author: Davis Vaughan [aut, cre],
  RStudio [cph]
Maintainer: Davis Vaughan <davis@rstudio.com>
Repository: CRAN
Date/Publication: 2021-07-01 19:50:01 UTC
Built: R 4.1.0; x86_64-w64-mingw32; 2021-07-01 23:40:37 UTC; windows
Archs: x64
